Package: covafillr
Title: Local Polynomial Regression of State Dependent Covariates in
        State-Space Models
Version: 0.4.4
Date: 2019-11-22
Authors@R: person(c("Christoffer","Moesgaard"), "Albertsen",
	   email = "cmoe@aqua.dtu.dk", role = c("aut", "cre"),
	   comment = c(ORCID = "0000-0003-0088-4363"))
Maintainer: Christoffer Moesgaard Albertsen <cmoe@aqua.dtu.dk>
Description: Facilitates local polynomial regression for state dependent covariates in state-space models. The functionality can also be used from 'C++' based model builder tools such as 'Rcpp'/'inline', 'TMB', or 'JAGS'.
BugReports: https://github.com/calbertsen/covafillr/issues
URL: https://github.com/calbertsen/covafillr
Depends: R (>= 3.0.0)
Imports: methods, stats, Rcpp (>= 0.11.0)
LinkingTo: RcppEigen
Suggests: TMB, rjags, inline, ggplot2
License: BSD_2_clause + file LICENSE
LazyData: true
Biarch: true
NeedsCompilation: yes
RoxygenNote: 7.0.0
Packaged: 2019-11-22 09:45:30 UTC; cmoe
Author: Christoffer Moesgaard Albertsen [aut, cre]
    (<https://orcid.org/0000-0003-0088-4363>)
Repository: CRAN
Date/Publication: 2019-11-22 10:40:09 UTC
Built: R 4.3.3; aarch64-apple-darwin20; 2025-01-24 06:04:48 UTC; unix
Archs: covafillr.so.dSYM
