# Generated by roxygen2: do not edit by hand

export(covafill)
export(covatree)
export(cxxFlags)
export(kde)
export(loadJAGSModule)
export(stat_covafill)
export(suggestBandwith)
exportClasses(covafill)
exportClasses(covatree)
importFrom(Rcpp,Rcpp.plugin.maker)
importFrom(methods,new)
importFrom(methods,setRefClass)
importFrom(stats,IQR)
importFrom(stats,qnorm)
importFrom(stats,sd)
useDynLib(covafillr, .registration = TRUE, .fixes = "C_")
